#
# random set cover problem
#

from coopr.pyomo import *
import math
import random

random.seed(9090)

model = ConcreteModel()


#
# m - number of elements
#
model.m = Param(within=PositiveIntegers, default=100)
#
# n - number of sets
#
model.n = Param(within=PositiveIntegers, default=200)
#
# p - max number elements per set
#
model.pr = Param(default=1.0/math.sqrt(value(model.m)), within=PercentFraction)

model.I = RangeSet(1, model.m)
model.J = RangeSet(1, model.n)

#
# = setof{i in J, j in 1 .. p} (i, ceil(Uniform(0,m)));
#
def S_rule(model):
    pr = value(model.pr)
    ans = set()
    for j in model.J:
        for i in model.I:
            if random.uniform(0,1) < pr:
                ans.add( (i, j) )
    return ans

model.S = Set(dimen=2, initialize=S_rule)

#
# weights
#
model.w = Param(model.J, within=NonNegativeIntegers, initialize=1)

model.x = Var(model.J, within=Binary)


#
# Objective
#
def cost_rule(instance):
    return summation(instance.w, instance.x)

model.cost = Objective(rule=cost_rule)

#
# Constraint
#
def cover_rule(instance, i):
    expr = 0
    for j in instance.J:
        if (i,j) in instance.S:
            expr += instance.x[j]
    if expr is 0:
        return Constraint.Skip
    return expr >= 1

model.cover = Constraint(model.I, rule=cover_rule)
